// RF24Network - Version: 1.0.8
#include <RF24Network.h>
#include <RF24Network_config.h>
#include <Sync.h>

// RF24 - Version: 1.3.1
#include <RF24.h>
#include "RF24_config.h"
#include <nRF24L01.h>
#include <printf.h>

#include <IRremote.h>

RF24 radio(7,8);                    // nRF24L01(+) radio attached using Getting Started board 
RF24Network network(radio);          // Network uses that radio

const uint16_t this_node = 01;        // Address of our node in Octal format
const uint16_t master_device = 00;        // Address of master device in Octal format

int time_delay = 100;

#define lightPin 4    // Light
#define pirPin 2      // PIR

IRsend irsend;
RF24NetworkHeader header;

struct action_to_slave {
  int message;                // 1-> IR code message || 2-> light message
  long IR_code;               // The IR code
  decode_type_t IR_protocol;  // The IR protocol
  int IR_length;              // The IR length
};

boolean light_on = false;
long last_check = 0;

void IR_Transmitt (long ir_code, decode_type_t ir_protocol, int ir_length_code) {
  for (int i = 0; i < 3; i++) {
    switch (ir_protocol) {
      case NEC:
        irsend.sendNEC(ir_code, ir_length_code);
        break;
      case SONY:
        irsend.sendSony(ir_code, ir_length_code);
        break;
      case RC5:
        irsend.sendRC5(ir_code, ir_length_code);
        break;
      case RC6:
        irsend.sendRC6(ir_code, ir_length_code);
        break;
      case LG:
        irsend.sendLG(ir_code, ir_length_code);
        break;
      case JVC:
        irsend.sendJVC(ir_code, ir_length_code, true);
        break;
      case WHYNTER:
        irsend.sendWhynter(ir_code, ir_length_code);
        break;
      case SAMSUNG:
        irsend.sendSAMSUNG(ir_code, ir_length_code);
        break;
      case SHARP:
        irsend.sendSharpRaw(ir_code, ir_length_code);
        break;
      case DISH:
        irsend.sendDISH(ir_code, ir_length_code);
        break;
      case DENON:
        irsend.sendDenon(ir_code, ir_length_code);
        break;
      case LEGO_PF:
        irsend.sendLegoPowerFunctions(ir_code, ir_length_code);
        break;
    }
    delay(40);
  }
}

void setup(void)
{
  radio.begin();
  radio.setPALevel(RF24_PA_MIN);
  radio.setDataRate(RF24_250KBPS);
  network.begin(/*channel*/ 70, /*node address*/ this_node);
  pinMode(lightPin, OUTPUT);
  pinMode(pirPin, INPUT);
}

void loop() {
  network.update();                               // Check the network regularly    
  while ( network.available() ) {                 // Is there any incoming data
    
    action_to_slave action_to_slave_t;
    network.read(header,&action_to_slave_t,sizeof(action_to_slave_t));
    if (action_to_slave_t.message == 1) {      // IR CODE action
      IR_Transmitt(action_to_slave_t.IR_code, action_to_slave_t.IR_protocol, action_to_slave_t.IR_length);
    }
    if (action_to_slave_t.message == 2) {      // LIGHT action
      if (not light_on) {
        digitalWrite(lightPin, HIGH);
        light_on = true;
    } else {
        digitalWrite(lightPin, LOW);
        light_on = false;
      }
    }
    if (action_to_slave_t.message == 3) {      // LIGHT OFF
      digitalWrite(lightPin, LOW);
      light_on = false;
    }
  }
  
  int pirValue = digitalRead(pirPin);
  if (pirValue == 1) {
    long time_now = millis();
    if (time_now - last_check < 1000) return;
    last_check = millis();
    header.to_node = master_device;
    bool message_sent = network.write(header,&this_node,sizeof(this_node));
  }
  
  delay(time_delay);
}
